#include "llvm/Analysis/BasicAliasAnalysis.h"
#include "llvm/Transforms/Scalar.h"
#include "llvm/Transforms/Scalar/GVN.h"
#include "llvm/IR/LegacyPassManager.h"
#include "KaleidoscopeJIT.h"
using namespace llvm;
using namespace llvm::orc;

extern LLVMContext TheContext;
extern std::unique_ptr<Module> TheModule;
extern std::unique_ptr<KaleidoscopeJIT> TheJIT;
std::unique_ptr<legacy::FunctionPassManager> TheFPM;

void InitializeModuleAndPassManager(void) {
    TheModule = llvm::make_unique<Module>("my cool jit", TheContext);
    TheModule->setDataLayout(TheJIT->getTargetMachine().createDataLayout());

    TheFPM = llvm::make_unique<legacy::FunctionPassManager>(TheModule.get());

    // Provide basic AliasAnalysis support for GVN.
    TheFPM->add(createBasicAAWrapperPass());
    // Do simple "peephole" optimizations and bit-twiddling optzns.
    TheFPM->add(createInstructionCombiningPass());
    // Reassociate expressions.
    TheFPM->add(createReassociatePass());
    // Eliminate Common SubExpressions.
    TheFPM->add(createGVNPass());
    // Simplify the control flow graph (deleting unreachable blocks, etc).
    TheFPM->add(createCFGSimplificationPass());

    TheFPM->doInitialization();
}

